#ifndef I2C_H
#define I2C_H

#include "sdk_errors.h"
#include "nrf_drv_twi.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief I2C instance definition.
 * @param[in] _name       Name of static variable to init.
 * @param[in] _i2cIfaceId Number of hw interface.
*/
#define I2C_DEF(_name, _i2cIfaceId)                                      \
static const i2c_instance_t _name = NRF_DRV_TWI_INSTANCE(_i2cIfaceId)

typedef nrf_drv_twi_t i2c_instance_t;

/**
 * @brief Function for I2C initialization.
 * @param[in] i2c         I2C instance.
 * @param[in] config      I2C configuration structure.
 *
 * @return                NRF_SUCCESS on successful operation, otherwise an error code of nrf_drv_twi_init().
 */
ret_code_t i2c_init(const i2c_instance_t *i2c, const nrf_drv_twi_config_t *config);

/**
 * @brief Function for detecting I2C device on bus.
 * @param[in] i2c         I2C instance.
 * @param[in] address     I2C destination address.
 *
 * @return                NRF_SUCCESS on successful operation, otherwise an error code of nrf_drv_twi_init().
 */
ret_code_t i2c_ping(const i2c_instance_t *i2c, uint8_t address);

/**
 * @brief Function for writing array of bytes via I2C.
 *
 * @param[in] i2c         I2C instance.
 * @param[in] address     I2C destination address.
 * @param[in] data        Pointer to array of bytes to send.
 * @param[in] numBytes    Count of bytes.
 *
 * @return                NRF_SUCCESS on successful operation, otherwise an error code of nrf_drv_twi_tx().
 */
ret_code_t i2c_writeBytes(const i2c_instance_t *i2c, uint8_t address, 
    const uint8_t const *data, uint8_t numBytes, bool writeStopBitFlag);

/**
 * @brief Function for reading array of bytes via I2C.
 *
 * @param[in] i2c         I2C instance.
 * @param[in] address     I2C destination address.
 * @param[out] buffer     Pointer to buffer of received data.
 * @param[in] numBytes    Bytes to read.
 *
 * @return                NRF_SUCCESS on successful operation, otherwise an error code of nrf_drv_twi_rx().
 */
ret_code_t i2c_readBytes(const i2c_instance_t *i2c, uint8_t address,
   uint8_t *buffer, uint8_t numBytes);
    

#ifdef __cplusplus
}
#endif

#endif /* I2C_H */
